/***************************************************************

  BayCom(R)        Packet-Radio fuer IBM PC

  OpenBayCom-Mailbox

  -----------------------------------------------------
  Betriebssystemabhaengiger Rahmen fuer Timer-Interrupt
  -----------------------------------------------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

#include "baycom.h"
#ifdef _WIN32
#include "windows.h"
#endif

/*---------------------------------------------------------------------------*/

#ifdef __DOS16__
static void interrupt far(far*oldvec)(...);
#endif

static void (*timer_callback)(void);

/*---------------------------------------------------------------------------*/

#ifdef _WIN32
void CALLBACK win_handler(UINT IDEvent,UINT uReserved,
                 DWORD  dwUser, DWORD  dwReserved1, DWORD  dwReserved2)
#else
#ifdef __UNIX__
static void timer_handler (int)
#else
static void interrupt far timer_handler (...)
#endif
#endif
{ static char recursive = 0;
#ifdef __DOS16__
  (oldvec)();
  disable ();
#endif
  if ( ! recursive)
  { recursive = 1;
    enable ();
    (*timer_callback)();
    recursive = 0;
  }
}

/*---------------------------------------------------------------------------*/

#ifdef __DPMI32__
#define PUSH_EBP __emit__(0x55)
#define PUSH_EAX __emit__(0x50)
#define PUSH_EBX __emit__(0x53)
#define PUSH_ECX __emit__(0x51)
#define PUSH_EDX __emit__(0x52)
#define PUSH_ESI __emit__(0x56)
#define PUSH_EDI __emit__(0x57)
#define PUSH_DS  __emit__(0x1E)
#define PUSH_ES  __emit__(0x06)

#define POP_ES   __emit__(0x07)
#define POP_DS   __emit__(0x1F)
#define POP_EDI  __emit__(0x5F)
#define POP_ESI  __emit__(0x5E)
#define POP_EDX  __emit__(0x5A)
#define POP_ECX  __emit__(0x59)
#define POP_EBX  __emit__(0x5B)
#define POP_EAX  __emit__(0x58)
#define POP_EBP  __emit__(0x5D)
#define IRETD    __emit__(0xCF)
#define NOP      __emit__(0x90)

short unsigned dpmi32_ds;
void timer_dpmi32(void)
{ PUSH_EBP;
  PUSH_EAX;
  PUSH_EBX;
  PUSH_ECX;
  PUSH_EDX;
  PUSH_ESI;
  PUSH_EDI;
  PUSH_DS;
  PUSH_ES;
  _AX=_CS;
  _DS=_AX;
  _AX=dpmi32_ds;
  _DS=_AX;
  timer_handler();
  POP_ES;
  POP_DS;
  POP_EDI;
  POP_ESI;
  POP_EDX;
  POP_ECX;
  POP_EBX;
  POP_EAX;
  POP_EBP;
  IRETD;
}
#endif

/*---------------------------------------------------------------------------*/

void ad_settimer (int on, void (*callback) (void))
{
  timer_callback = callback;
#ifdef _WIN32
  static UINT ev_id;
  static int ison = 0;
  if (on)
  { if ( ! ison)
    { ison = 1;
      ev_id = timeSetEvent (55, 5, (LPTIMECALLBACK) win_handler, 0x0, TIME_PERIODIC);
    }
  }
  else if (ison)
  {
    timeKillEvent (ev_id);
    ison = 0;
  }
#elif defined(__DPMI32__)
  static unsigned long oldisr;
  static unsigned long oldiseg;
  static int ison = 0;

  if (on)
  { if ( ! ison)
    { ison = 1;
      _AX = _DS;
      dpmi32_ds = _AX;
      _EBX = 0x1c;
      _EAX = 0x204;
      geninterrupt (0x31);
      oldiseg = _ECX;
      oldisr = _EDX;
      _EBX = 0x1c;
      _CX = _CS;
      _EDX = (long) timer_dpmi32;
      _EAX = 0x205;
      geninterrupt (0x31);
    }
  }
  else
  if (ison)
  {
    _EBX = 0x1c;
    _ECX = oldiseg;
    _EDX = oldisr;
    _EAX = 0x205;
    geninterrupt (0x31);
    ison = 0;
  }
#elif defined(__MSDOS__)
  static int ison = 0;
  if (on)
  { if ( ! ison)
    { ison = 1;
      oldvec = getvect (0x1c);
      setvect (0x1c, timer_handler);
    }
  }
  else
  if (ison)
  { setvect (0x1c, oldvec);
    ison = 0;
  }
#else
  static int first = 1;
  if (first && on)
  { static struct itimerval rttimer;
    static struct itimerval old_rttimer;
    struct sigaction sa;

    first = 0;
    memset (&rttimer, 0, sizeof (struct itimerval));
    rttimer.it_value.tv_sec     = 0;
    rttimer.it_value.tv_usec    = MS_PER_TIC * 1000;
    rttimer.it_interval.tv_sec  = 0;
    rttimer.it_interval.tv_usec = MS_PER_TIC * 1000;
    sa.sa_handler = timer_handler;
    sa.sa_flags = SA_RESTART | SA_NOMASK;
    sigaction (SIGALRM, &sa, NULL);
    setitimer (ITIMER_REAL, &rttimer, &old_rttimer);
  }
#endif
}

