/***************************************************************

  BayCom(R)        Packet-Radio fuer IBM PC

  OpenBayCom-Mailbox

  --------------------------------------------------------------
  Timer-Interrupt (Systemunabhaengiger Teil, siehe ad_timer.cpp)
  --------------------------------------------------------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//990404 OE3DZW added minwatch - find blocking code

#include "baycom.h"

#define CPUVALUES 8

/*---------------------------------------------------------------------------*/

long systic = 0;
int lastjob;     // is written by showtasknum in tasking.cpp

/*---------------------------------------------------------------------------*/

static char idletics [CPUVALUES] = "";
static char updatics [CPUVALUES] = "";
static char runtics  [CPUVALUES] = "";
static char t_index = 0;
static unsigned int watchdog = 1000;
static unsigned int minwatch = 0;  //min watchdog
static char minwcmd [60] = ""; //command with min. watchdog timer

/*---------------------------------------------------------------------------*/

// Callback from ad_settimer
void mbtimer (void)
{ static char l_idletics = 0;
  static char l_updatics = 0;
  static char l_runtics  = 0;
  static char l_tics     = 0;

  systic++;
  switch (lastjob)
  { case 'D':
    case 'I': l_idletics++; break;
    case 'N':
    case 'R': l_runtics++;  break;
    case 'M':
    case 'U': l_updatics++; break;
  }
  l_tics++;
  if (l_tics >= 100)
  { idletics [t_index] = l_idletics;
    updatics [t_index] = l_updatics;
    runtics  [t_index] = l_runtics;
    l_tics =
    l_idletics =
    l_updatics =
    l_runtics = 0;
    t_index = (t_index + 1) % CPUVALUES;
  }
  watchdog--;          // watchdog was set to a positive value
  if ( ! watchdog)         // now it has expired
  { if (m.watchdog & 2) // is sw-watchdog enabled?
    { if (m.watchdog & 4) post_mortem_dump ("watchdog");     // is post mortem dump enabled?
      disable();            // be careful...
      coldstart();        // hope this still runs...
    }
  }
  if (t && ( ! minwatch || watchdog < minwatch))
  { minwatch = watchdog;
    strncpy (minwcmd, t->lastcmd, sizeof (minwcmd) - 1);
    minwcmd [sizeof (minwcmd) - 1] = 0;  //add terminating 0-byte
  }
}

/*---------------------------------------------------------------------------*/

int cpuload (void)
{ int ret = 0;

  for (unsigned int i = 0; i < CPUVALUES; i++)
  { ret += runtics[i];
    ret += updatics[i];
  }
  return ret / CPUVALUES;
}

/*---------------------------------------------------------------------------*/

static void near load_line (char *val, char *nam)
{ int i = t_index, x = t_index;
  putf ("%s ", nam);
  do
  { i--;
    if (i < 0) i = CPUVALUES - 1;
    putf ("%3d ", val[i]);
  } while (i != x);
  putf ("%\n");
}

/*---------------------------------------------------------------------------*/

void put_cpuload (void)
{ putf ("CPU load in 5sec-intervals:\n");
  putf ("Sample:  now -5s -10s ...\n");
  load_line (runtics, "Running:");
  load_line (updatics, "Screen: ");
  load_line (idletics, "Idle:   ");
  putf ("Max. tics: %u at \"%s\"\n", 5460 - minwatch, minwcmd);
}

/*---------------------------------------------------------------------------*/

static void clear_watchdog (void)
{
  set_watchdog (0);
}

/*---------------------------------------------------------------------------*/

void set_watchdog (int on)
{ static int first = 1;
  if (on && first)
  { atexit (clear_watchdog);
    first = 0;
  }
  if (on == 2) watchdog = 32760;        // 30min for DOS calls etc
  else watchdog = 5460;                 // 2min for normal operation
  ad_settimer (on, mbtimer);
}

/*---------------------------------------------------------------------------*/
